local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

require "ndi_decoding"

local RATE_HD = 0
local RATE_SD = 1
local RATE_3G = 2
local RATE_4K = 3 --TODO: Not ensured.
local RATE_6G = 4
local RATE_12G = 5

local FMT_525i = 0
local FMT_625i = 1

local FMT_720p50 = 0
local FMT_1080pSF = 1
local FMT_1080i60 = 2
local FMT_1080i50 = 3
local FMT_1080p30 = 4
local FMT_1080p25 = 5
local FMT_1080p24 = 6
local FMT_720p60 = 7

local FMT_1080p60 = 4
local FMT_1080p50 = 5

local NTSC_CLK = 1
local DEFAULT_CLK = 0

local formatMap = {
	-- 4K formats.
	["4k60"]		=	{	rate = RATE_12G,	fmt = FMT_1080p60,		clk = DEFAULT_CLK	},
	["4k59"]		=	{	rate = RATE_12G,	fmt = FMT_1080p60,		clk = NTSC_CLK  	},
	["4k50"]		=	{	rate = RATE_12G,	fmt = FMT_1080p50,		clk = DEFAULT_CLK	},
	["4k30"]		=	{	rate = RATE_6G,		fmt = FMT_1080p30,		clk = DEFAULT_CLK	},
	["4k29"]		=	{	rate = RATE_6G,		fmt = FMT_1080p30,		clk = NTSC_CLK	    },
	["4k25"]		=	{	rate = RATE_6G,		fmt = FMT_1080p25,		clk = DEFAULT_CLK	},
	["4k24"]		=	{	rate = RATE_6G,		fmt = FMT_1080p24,		clk = DEFAULT_CLK	},
	["4k23"]		=	{	rate = RATE_6G,		fmt = FMT_1080p24,		clk = NTSC_CLK  	},
	-- 3G formats
	["1080p60"]		=	{	rate = RATE_3G,		fmt = FMT_1080p60,		clk = DEFAULT_CLK	},
	["1080p59"]		=	{	rate = RATE_3G,		fmt = FMT_1080p60,		clk = NTSC_CLK		},
	["1080p50"]		=	{	rate = RATE_3G,		fmt = FMT_1080p50,		clk = DEFAULT_CLK	},
	["1080p30"]		=	{	rate = RATE_HD,		fmt = FMT_1080p30,		clk = DEFAULT_CLK	},
	["1080p29"]		=	{	rate = RATE_HD,		fmt = FMT_1080p30,		clk = NTSC_CLK		},
	["1080p25"]		=	{	rate = RATE_HD,		fmt = FMT_1080p25,		clk = DEFAULT_CLK	},
	["1080p24"]		=	{	rate = RATE_HD,		fmt = FMT_1080p24,		clk = DEFAULT_CLK	},
	["1080p23"]		=	{	rate = RATE_HD,		fmt = FMT_1080p24,		clk = NTSC_CLK		},
	["1080i60"]		=	{	rate = RATE_HD,		fmt = FMT_1080i60,		clk = DEFAULT_CLK	},
	["1080i59"]		=	{	rate = RATE_HD,		fmt = FMT_1080i60,		clk = NTSC_CLK		},
	["1080i50"]		=	{	rate = RATE_HD,		fmt = FMT_1080i50,		clk = DEFAULT_CLK	},
	["720p60"]		=	{	rate = RATE_HD,		fmt = FMT_720p60,		clk = DEFAULT_CLK	},
	["720p59"]		=	{	rate = RATE_HD,		fmt = FMT_720p60,		clk = NTSC_CLK		},
	["720p50"]		=	{	rate = RATE_HD,		fmt = FMT_720p50,		clk = DEFAULT_CLK	},
	["480i"]		=	{	rate = RATE_SD,		fmt = FMT_525i,			clk = DEFAULT_CLK	},
	["576i"]		=	{	rate = RATE_SD,		fmt = FMT_625i,			clk = DEFAULT_CLK	},
	["525i"]		=	{	rate = RATE_SD,		fmt = FMT_525i,			clk = DEFAULT_CLK	},
	["625i"]		=	{	rate = RATE_SD,		fmt = FMT_625i,			clk = DEFAULT_CLK	}
}

local function CreateFPGA_Tx()
-------------------------------------------------------
--{ The constructor wrap function.
-------------------------------------------------------

local _M = {
	first_run = true
}
----
function _M:Init()
	--TODO: nothing
	return true
end
----
function _M:Step()
	--TODO: nothing
end
----
function _M:UpdateVideoFormat( fmtInfo )
	if type(fmtInfo) ~= "table" or not fmtInfo.id then
		_dbg.Error("SDI-FPGA", "Invalid format information")
		return false
	end
	_dbg.Info("SDI-FPGA", "Configure current format id:", fmtInfo.id )
	local fmtSet = formatMap[fmtInfo.id]
	if not fmtSet then
		_dbg.Error("SDI-FPGA", "Unsupported format ID:", fmtInfo.id, ", configuration is ignored." )
		return false
	end
	local regVal = (fmtSet.rate << 14) | (fmtSet.fmt << 11) | (fmtSet.clk << 10)
	local regVal0

	if fmtInfo.deInterlace then
		regVal0 = regVal
		regVal = regVal | 0x80
	else
		regVal0 = regVal | 0x80
	end

	_dbg.Info("SDI-FPGA", "Format register value is:", string.format("%08x", regVal))
	NDI.DECODING.write_reg( 0x280, 0x0e, regVal )

	--A temporary FPGA bug fix.
	if fmtInfo.interlaced and _M.first_run then
		_M.first_run = false
		coaf.Sleep(0.1)
		NDI.DECODING.write_reg( 0x280, 0x0e, regVal0 )
		coaf.Sleep(0.1)
		NDI.DECODING.write_reg( 0x280, 0x0e, regVal )
	end

	return true
end

----

function _M:UpdateAudioFormat( fmtInfo )
	--TODO: nothing.
	return true
end

_M:Init()

return _M

-------------------------------------------------------
--} End of the constructor wrap function.
-------------------------------------------------------
end

return CreateFPGA_Tx
